
import com.ewon.ewonitf.IOServer;
import com.ewon.ewonitf.IOTag;
import com.ewon.ewonitf.IOValue;
import java.util.Vector;

  public class IOserver extends IOServer implements Runnable {
    
   //Set your Global variables here
   long pollrate = 2500;
   
   //Growable array of Object used to store your Tags List
   Vector MyTagsList = new Vector();
   Vector TagsIOName = new Vector();
   
         
   public void onRegister() throws Exception
   {
       //setJavaReplyTimeout(1000000);   //Let Java freeze the eWON while debugging.
   }
      
    public IOserver()
   {
      super();   
      
      //Type your IO Server Name here.  This is the name used in the eWON IO Server and Tag webpages
      setServerName("NewIOSrv");     
   }
       
    public void onPassConfig(boolean applyConfig, boolean checkConfig) throws Exception 
    {
        //Executed when passing a new IOServer configuration [ eWON Webpage: System Setup \ IOServer \ NewIOServerName(See above) ] 
        //Here, in the example, "pollrate" is the parameter name and "5000" is the default value in case the parameter is not defined.
        
        //The OnPassConfig function is executed twice, once for checking the new parameter values (checkConfig), once for applying new values (applyConfig)
        if (checkConfig)  
        {
            try
            {
                
              long temp_pollrate = Long.parseLong(getConfigParam("pollrate","5000"));
                     
              if (temp_pollrate < 500)
                throw new Exception("pollrate cannot be lower than 500 milliseconds");
            } 
            catch(Exception ex)
            {
                //Print exception created above in Events Log of eWON
                throw new Exception("Invalid pollrate value + (Exception : " + ex.getMessage());
            }
        }
        
        if (applyConfig)
        {       
            pollrate = Long.parseLong(getConfigParam("pollrate","5000"));
            System.out.println("DEBUG (onPassConfig): New value for pollrate parameter, " + pollrate + " ms"); 
        }
      
    }
    
 
    public IOValue onGetIo(IOTag ioTag) throws Exception
    {        
        //Firmware requests the value of the Tag. Typically at the Tag init or when doing Historical logging based on Time interval
                
        System.out.println("DEBUG (onGetIo): Tag = " + ioTag.getIoName());
        return ioTag.getIoData();
        
    }
           
    public IOTag onGetIoInfo(String topicName, String ioName) throws Exception {
      
        //To validate the Topic and Address of Tag and set its default value type (Float, Int, Bool) as well as it is an Output or not.
        //If the Topic and Address are not OK, throw an Exception in your code.
        
        if ("".equals(ioName))
            throw new Exception("Tag Address cannot be empty");
        
        for (int i=0;i<TagsIOName.size();i++)
        {
            String t = (String) TagsIOName.elementAt(i);
            System.out.println("DEBUG (onGetIoInfo - t): Tag= " + t);
            if (t.equals(ioName))
                 throw new Exception("Duplicate Tag");
        }
        
        TagsIOName.addElement(ioName);
        
        IOTag ioTag = new IOTag(topicName, ioName, IOValue.DATATYPE_FLOAT32, true);  
        System.out.println("DEBUG (onGetIoInfo): Topic = " + topicName + ", ioName = " + ioName);
        
        return ioTag;
   }


     public void onAdviseIo(IOTag ioTag) throws Exception 
     {    
         //A Tag has been added and validated (See onGetIoInfo() )
       
         System.out.println("DEBUG (onAdviseIo): Tag= " + ioTag.getIoName());
         
         //Store the tag internally in your IO server Tags List to monitor it from the run() function
         ioTag.setQuality(IOTag.QUALITY_BAD);
         postQuality(ioTag);
         MyTagsList.addElement(ioTag);
     }

     public void onUnadviseIo(IOTag ioTag) throws Exception 
     {
         //Tag has been removed from Tags list
         //Remove the tag from your IO server and monitor it
         System.out.println("DEBUG (onUnAdviseIo): Tag= " + ioTag.getIoName());         
        
         //Remove the Tag from the Tags List
         MyTagsList.removeElement(ioTag);
     }
   
     public void onPutIo(IOTag ioTag, IOValue ioData) throws Exception
     {
         //User writes a new value to the Tag (Web page, Modbus,...)
         System.out.println("DEBUG (onPutIo): Tag= " + ioTag.getIoName() + ", value= " + ioData.getFloat32()); 
     }         
      
     public void run()
     {            
      //This is where you implement your communication process/loop.
      
      int Counter = 0;
      
      //Endless loop used to create your communication process (Use ETK APIs to handle serial or IP communication)           
      while(true)
       {
           
            try {
                Counter ++;
                System.out.println("Communication Cycle #" + Counter);
                
                for (int i=0;i<MyTagsList.size();i++)
                {
                    IOTag TagToProcess = (IOTag) MyTagsList.elementAt(i);
                    System.out.println("Processing Tag "  + TagToProcess.getIoName());
                    
                    //If Address of Tag is "counter", set the counter value to the Tag
                    if ("counter".equals(TagToProcess.getIoName()))
                    {
                        //Set new value in object
                        TagToProcess.setValueFloat32(Counter);
                        
                        //publish value to eWON Tag. Must only be executed when the value has really changed.  Keep posting the same value must be prevented.
                        postIoChange(TagToProcess);
                    }
                    
                       TagToProcess.setQuality(IOTag.QUALITY_GOOD);
                       postQuality(TagToProcess);
                }
                
                //Wait a certain time between each communication cycle --> See OnPassConfig()
                Thread.sleep(pollrate);
                
            } catch (Exception ex) {
                System.out.println("Main Loop Error : " + ex.getMessage());
            }
       }
     }
     
   
  }
  
    

